@@ -620,7 +620,7 @@ install_menuitem_hint_to_statusbar(GnomeUIInfo* uiinfo, GtkStatusbar* bar)
 }
 
 /**
- * gnome_app_install_statusbar_menu_hints:
+ * gnome_app_install_statusbar_menu_hints
  * @bar: Pointer to #GtkStatusbar instance.
  * @uiinfo: #GnomeUIInfo for the menu to be changed.
  *
@@ -730,7 +730,7 @@ install_menuitem_hint_to_appbar(GnomeUIInfo* uiinfo, GnomeAppBar* bar)
 
 
 /**
- * gnome_app_ui_configure_configurable:
+ * gnome_app_ui_configure_configurable
  * @uiinfo: Pointer to GNOME UI menu/toolbar info
  *
  * Description:
@@ -822,7 +822,7 @@ gnome_app_ui_configure_configurable (GnomeUIInfo* uiinfo)
 
 
 /**
- * gnome_app_install_appbar_menu_hints:
+ * gnome_app_install_appbar_menu_hints
  * @appbar: An existing #GnomeAppBar instance.
  * @uiinfo: A #GnomeUIInfo array of a menu for which hints will be set.
  *
@@ -878,7 +878,7 @@ gnome_app_install_appbar_menu_hints (GnomeAppBar* appbar,
 
 
 /**
- * gnome_app_install_menu_hints:
+ * gnome_app_install_menu_hints
  * @app: An existing #GnomeAppBar instance.
  * @uiinfo: A #GnomeUIInfo array of a menu for which hints will be set.
  *
@@ -1023,7 +1023,7 @@ create_menu_item (GtkMenuShell       *menu_shell,
 		GtkWidget *widget;
 
 		/* build up the menu item path */
-		gstring = g_string_new ("");
+		gstring = g_string_new (NULL);
 		widget = uiinfo->widget;
 		while (widget) {
 			if (GTK_IS_MENU_ITEM (widget)) {
@@ -1222,7 +1222,7 @@ do_ui_signal_connect (GnomeUIInfo        *uiinfo,
 
 
 /**
- * gnome_app_fill_menu:
+ * gnome_app_fill_menu
  * @menu_shell: A #GtkMenuShell instance (a menu bar).
  * @uiinfo: A pointer to the first element in an array of #GnomeUIInfo
  * structures. The last element of the array should have a type of
@@ -1234,7 +1234,7 @@ do_ui_signal_connect (GnomeUIInfo        *uiinfo,
  *
  * Description:
  * Fills the specified @menu_shell with items created from the specified
- * @uiinfo, inserting them from the item number @pos on.  The @accel_group
+ * @uiinfo, inserting them from the item number @pos on.  The @accel_ group
  * will be used as the accel group for all newly created sub menus and serves
  * as the global accel group for all menu item hotkeys. If it is passed as
  * %NULL, global hotkeys will be disabled.
@@ -1268,7 +1268,7 @@ gnome_app_fill_menu (GtkMenuShell  *menu_shell,
 
 
 /**
- * gnome_app_fill_menu_with_data:
+ * gnome_app_fill_menu_with_data
  * @menu_shell: A #GtkMenuShell instance (a menu bar).
  * @uiinfo: A pointer to the first element in an array of #GnomeUIInfo
  * structures. The last element of the array should have a type of
@@ -1356,7 +1356,7 @@ menus_have_tearoff_changed_notify(GConfClient            *client,
 
 
 /**
- * gnome_app_fill_menu_custom:
+ * gnome_app_fill_menu_custom
  * @menu_shell: A #GtkMenuShell instance (a menu bar).
  * @uiinfo: A pointer to the first element in an array of #GnomeUIInfo
  * structures. The last element of the array should have a type of
@@ -1498,7 +1498,7 @@ gnome_app_fill_menu_custom (GtkMenuShell       *menu_shell,
 
 
 /**
- * gnome_app_create_menus:
+ * gnome_app_create_menus
  * @app: A #GnomeApp instance representing the current application.
  * @uiinfo: The first in an array #GnomeUIInfo instances containing the menu
  * data.
@@ -1528,7 +1528,7 @@ gnome_app_create_menus (GnomeApp *app, GnomeUIInfo *uiinfo)
 
 
 /**
- * gnome_app_create_menus_interp:
+ * gnome_app_create_menus_interp
  * @app: A #GnomeApp instance, representing the current application.
  * @uiinfo: The first item in an array of #GnomeUIInfo structures describing
  * the menu bar.
@@ -1562,7 +1562,7 @@ gnome_app_create_menus_interp (GnomeApp *app, GnomeUIInfo *uiinfo,
 
 
 /**
- * gnome_app_create_menus_with_data:
+ * gnome_app_create_menus_with_data
  * @app: A #GnomeApp instance representing the current application.
  * @uiinfo: The first in an array #GnomeUIInfo instances containing the menu
  * data.
@@ -1570,7 +1570,7 @@ gnome_app_create_menus_interp (GnomeApp *app, GnomeUIInfo *uiinfo,
  * function.
  *
  * Identical to gnome_app_create_menus(), except that @user_data is passed to
- * all the callback functions when signals are emitted.
+ * all the callback funtions when signals are emitted.
  **/
 
 void
@@ -1638,7 +1638,7 @@ gnome_app_set_tearoff_menu_titles(GnomeApp *app, GnomeUIInfo *uiinfo,
 
 
 /**
- * gnome_app_create_menus_custom:
+ * gnome_app_create_menus_custom
  * @app: A #GnomeApp instance representing the current application.
  * @uiinfo: The first in an array #GnomeUIInfo instances containing the menu
  * data.
@@ -1782,7 +1782,7 @@ create_radio_toolbar_items (GtkToolbar *toolbar, GnomeUIInfo *uiinfo,
 
 
 /**
- * gnome_app_fill_toolbar:
+ * gnome_app_fill_toolbar
  * @toolbar: A #GtkToolbar instance.
  * @uiinfo: An array of #GnomeUIInfo structures containing the items for the
  * toolbar.
@@ -1790,7 +1790,7 @@ create_radio_toolbar_items (GtkToolbar *toolbar, GnomeUIInfo *uiinfo,
  * (or %NULL).
  *
  * Fills @toolbar with buttons specified in @uiinfo. If @accel_group is not
- * %NULL, the items' accelerator keys are put into it.
+ * %NULL, the items' accelrator keys are put into it.
  **/
 
 void
@@ -1814,7 +1814,7 @@ gnome_app_fill_toolbar (GtkToolbar *toolbar, GnomeUIInfo *uiinfo,
 
 
 /**
- * gnome_app_fill_toolbar_with_data:
+ * gnome_app_fill_toolbar_with_data
  * @toolbar: A #GtkToolbar instance.
  * @uiinfo: An array of #GnomeUIInfo structures containing the items for the
  * toolbar.
@@ -1847,7 +1847,7 @@ gnome_app_fill_toolbar_with_data (GtkToolbar *toolbar, GnomeUIInfo *uiinfo,
 
 
 /**
- * gnome_app_fill_toolbar_custom:
+ * gnome_app_fill_toolbar_custom
  * @toolbar: A #GtkToolbar instance.
  * @uiinfo: An array of #GnomeUIInfo structures containing the items for the
  * toolbar.
@@ -1906,7 +1906,7 @@ gnome_app_fill_toolbar_custom (GtkToolbar *toolbar, GnomeUIInfo *uiinfo,
 }
 
 /**
- * gnome_app_create_toolbar:
+ * gnome_app_create_toolbar
  * @app: A #GnomeApp instance.
  * @uiinfo: A #GnomeUIInfo array specifying the contents of the toolbar.
  *
@@ -1934,7 +1934,7 @@ gnome_app_create_toolbar (GnomeApp *app, GnomeUIInfo *uiinfo)
 }
 
 /**
- * gnome_app_create_toolbar_interp:
+ * gnome_app_create_toolbar_interp
  * @app: A #GnomeApp instance.
  * @uiinfo: A #GnomeUIInfo array specifying the contents of the toolbar.
  * @relay_func: Argument marshalling function.
@@ -1967,7 +1967,7 @@ gnome_app_create_toolbar_interp (GnomeApp *app, GnomeUIInfo *uiinfo,
 }
 
 /**
- * gnome_app_create_toolbar_with_data:
+ * gnome_app_create_toolbar_with_data
  * @app: A #GnomeApp instance.
  * @uiinfo: A #GnomeUIInfo array specifying the contents of the toolbar.
  * @user_data: Application specific data to be sent to each signal callback
@@ -1998,7 +1998,7 @@ gnome_app_create_toolbar_with_data (GnomeApp *app, GnomeUIInfo *uiinfo,
 }
 
 /**
- * gnome_app_create_toolbar_custom:
+ * gnome_app_create_toolbar_custom
  * @app: A #GnomeApp instance.
  * @uiinfo: A #GnomeUIInfo array specifying the contents of the toolbar.
  * @uibdata: A #GnomeUIBuilderData instance specifying the handlers to use for
@@ -2031,7 +2031,7 @@ gnome_app_create_toolbar_custom (GnomeApp *app, GnomeUIInfo *uiinfo, GnomeUIBuil
 }
 
 /**
- * g_strncmp_ignore_char:
+ * g_strncmp_ignore_char
  * @first: The first string to compare
  * @second: The second string to compare
  * @length: The length of the first string to consider (the length of
@@ -2042,9 +2042,9 @@ gnome_app_create_toolbar_custom (GnomeApp *app, GnomeUIInfo *uiinfo, GnomeUIBuil
  * Description:
  * Does a strcmp, only considering the first length characters of
  * first, and ignoring the ignored character in both strings.
- * Returns: -1 if first comes before second in lexicographical order, 0 if 
- * they're equivalent, and 1 if the second comes before the first in 
- * lexicographical order.
+ * Returns -1 if first comes before second in lexicographical order,
+ * Returns 0 if they're equivalent, and
+ * Returns 1 if the second comes before the first in lexicographical order.
  **/
 
 static gint
@@ -2071,7 +2071,7 @@ g_strncmp_ignore_char( const gchar *first, const gchar *second, gint length, gch
  */
 
 /**
- * gnome_app_find_menu_pos:
+ * gnome_app_find_menu_pos
  * @parent: Root menu shell widget containing menu items to be searched.
  * @path: Specifies the target menu item by menu path.
  * @pos: Used to hold the returned menu items' position.
@@ -2170,7 +2170,7 @@ gnome_app_find_menu_pos (GtkWidget *parent, const gchar *path, gint *pos)
 
 
 /**
- * gnome_app_remove_menus:
+ * gnome_app_remove_menus
  * @app: A #GnomeApp instance.
  * @path: A path to the menu item concerned.
  * @items: The number of items to remove.
@@ -2228,7 +2228,7 @@ gnome_app_remove_menus(GnomeApp *app, const gchar *path, gint items)
 
 
 /**
- * gnome_app_remove_menu_range:
+ * gnome_app_remove_menu_range
  * @app: A #GnomeApp instance.
  * @path: A path to the menu item concerned.
  * @start: An offset beyond the start of @path at which to begin removing.
@@ -2284,7 +2284,7 @@ gnome_app_remove_menu_range (GnomeApp *app, const gchar *path, gint start, gint
 }
 
 /**
- * gnome_app_insert_menus_custom:
+ * gnome_app_insert_menus_custom
  * @app: A #GnomeApp instance.
  * @path: A path to the menu item concerned.
  * @uiinfo: A #GnomeUIInfo array describing the menus.
@@ -2322,7 +2322,7 @@ gnome_app_insert_menus_custom (GnomeApp *app, const gchar *path,
 
 
 /**
- * gnome_app_insert_menus:
+ * gnome_app_insert_menus
  * @app: A #GnomeApp instance.
  * @path: A path to the menu item concerned.
  * @menuinfo: A #GnomeUIInfo array describing the menus.
@@ -2347,7 +2347,7 @@ gnome_app_insert_menus (GnomeApp *app,
 
 
 /**
- * gnome_app_insert_menus_with_data:
+ * gnome_app_insert_menus_with_data
  * @app: A #GnomeApp instance.
  * @path: A path to the menu item concerned.
  * @menuinfo: A #GnomeUIInfo array describing the menus.
@@ -2374,7 +2374,7 @@ gnome_app_insert_menus_with_data (GnomeApp *app, const gchar *path,
 
 
 /**
- * gnome_app_insert_menus_interp:
+ * gnome_app_insert_menus_interp
  * @app: A #GnomeApp instance.
  * @path: A path to the menu item concerned.
  * @menuinfo: A #GnomeUIInfo array describing the menus.
@@ -2422,6 +2422,7 @@ static GConfEnumStringPair toolbar_styles[] = {
         { GTK_TOOLBAR_ICONS, "icons" },
         { GTK_TOOLBAR_BOTH, "both" },
         { GTK_TOOLBAR_BOTH_HORIZ, "both_horiz" },
+	{ GTK_TOOLBAR_BOTH_HORIZ, "both-horiz" },
 	{ -1, NULL }
 };
 
@@ -2689,7 +2690,7 @@ button_press (GtkWidget *dock, GdkEventButton *event, gpointer user_data)
 }
 
 /**
- * gnome_app_setup_toolbar:
+ * gnome_app_setup_toolbar
  * @toolbar: Pointer to a #GtkToolbar instance.
  * @dock_item: Pointer to the #BonoboDockItem the toolbar is inside, or %NULL
  * for none.
